@@ -1,12 +1,26 @@
 package App::Mobirc::Plugin::Component::IRCClient;
 use strict;
-use App::Mobirc::Plugin;
+use MooseX::Plaggerize::Plugin;
 
 use POE;
 use POE::Sugar::Args;
-use POE::Component::IRC 5.88;
+use POE::Component::IRC;
 use POE::Filter::IRC::Compat;
 
+{
+    # HACK HACK HACK
+    # POE::Filter::IRC::Compat->get_one returns undef when invalid CTCP request.
+    # POE::Filter::Stackable dies when get the undef!
+    # see also http://rt.cpan.org/Ticket/Display.html?id=38773
+    my $meta = Class::MOP::Class->initialize('POE::Filter::IRC::Compat') or die "cannot get meta class of the POE::Filter::IRC::Compat";
+    $meta->add_around_method_modifier('get_one', sub {
+        my ($next, @args) = @_;
+        my $result = $next->(@args);
+        $result ? $result : [];
+    });
+    $meta->make_immutable;
+}
+
 use Encode;
 use Carp;
 
@@ -156,7 +170,7 @@ hook 'run_component' => sub {
     );
 
     $global_context->add_channel(
-        App::Mobirc::Model::Channel->new( name => U('*server*') ) );
+        App::Mobirc::Model::Channel->new( $global_context, U('*server*') ) );
 };
 
 # -------------------------------------------------------------------------
@@ -213,7 +227,8 @@ sub on_irc_join {
     my $channel = $poe->heap->{global_context}->get_channel($channel_name);
     unless ($channel) {
         $channel =
-          App::Mobirc::Model::Channel->new( name => $channel_name );
+          App::Mobirc::Model::Channel->new( $poe->heap->{global_context},
+            $channel_name, );
         $poe->heap->{global_context}->add_channel($channel);
     }
 
