@@ -1,7 +1,7 @@
 package App::Mobirc::Plugin::HTMLFilter::CompressHTML;
 # vim:expandtab:
 use strict;
-use App::Mobirc::Plugin;
+use MooseX::Plaggerize::Plugin;
 use App::Mobirc::Util;
 use App::Mobirc::Validator;
 
